/*
// File:		PrintMessage.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include <iostream>
#include "PrintMessage.hpp"
#include "StreamUtils.hpp"

void
PrintMessage(
		MessagePartEnumerator& inMessagePartEnumerator
	)
{
	//Print message subject field
	std::cout << "Subject:" << std::endl << "\t" << inMessagePartEnumerator.GetSubject() << std::endl << std::endl;

	//Print message contacts
	std::cout << "From:" << std::endl;
	Emai::ContactList contacts = inMessagePartEnumerator.GetContacts(EmaiFrom);
	EmaiUint32 count = contacts.GetCount();
	for (EmaiUint32 i = 0; i < count; ++i)
		std::cout << "\t\"" << contacts.GetName(i) << "\" " << contacts.GetAddress(i) << std::endl;
	std::cout << std::endl;

	std::cout << "To:" << std::endl;
	contacts = inMessagePartEnumerator.GetContacts(EmaiTo);
	count = contacts.GetCount();
	for (EmaiUint32 i = 0; i < count; ++i)
		std::cout << "\t\"" << contacts.GetName(i) << "\" " << contacts.GetAddress(i) << std::endl;
	std::cout << std::endl;

	//Print plain text body
	Emai::String text;
	inMessagePartEnumerator.GetText(text);
	std::cout << "Plain text body:" << std::endl << text << std::endl;

	//Print html body
	inMessagePartEnumerator.GetHtml(text);
	std::cout << "HTML text body:" << std::endl << text << std::endl;

	//Print attachments names and sizes
	EmaiUint32 attachmentsCount = inMessagePartEnumerator.GetAttachmentsCount();
	EmaiUint32 length = 0;
	if (attachmentsCount > 0)
	{
		std::cout << "Attachment:" << std::endl;
		Emai::UniString fileName;
		for (EmaiUint32 j = 0; j < attachmentsCount; j++)
		{
			inMessagePartEnumerator.GetAttachmentAtIndex(j, fileName, NULL, length);
			
			std::cout << "\tfile name: " << fileName << std::endl;
			std::cout << "\tfile size: " << length << std::endl;
		}
		std::cout << std::endl;
	}

	//Print inlines names, sizes and content IDs
	EmaiUint32 inlinesCount = inMessagePartEnumerator.GetInlinesCount();
	if (inlinesCount > 0)
	{
		std::cout << "Attachment inline:" << std::endl;
		Emai::UniString fileName;
		Emai::UniString contentID;
		for (EmaiUint32 j = 0; j < inlinesCount; j++)
		{
			inMessagePartEnumerator.GetInlineAtIndex(j, fileName, contentID, NULL, length);
			
			std::cout << "\tfile name: " << fileName << std::endl;
			std::cout << "\tcontent ID: " << contentID << std::endl;
			std::cout << "\tfile size: " << length << std::endl;
		}
		std::cout << std::endl;
	}


	std::cout << std::endl;
}
