/*
// File:		Pop3Callback.cpp
//
// Version:		mail.it library 1.0 sample file
//
// Copyright:	(C) 2007 Dacons LLP.
//
// Bugs:		If you have problems or suggestions, please contact
//				the Online Support at: http://www.dacons.net/support
*/

#include PCH_FILE_NAME
#include "Pop3Callback.hpp"

EmaiError
Pop3Callback::OnOperation(
		Emai::Pop3Session& inSession,
		EmaiUint32 inOperationCode
	)
{
	HWND hwnd = NULL;
	EmaiUint32 statSize = sizeof(HWND);
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(Pop3PercentText, &hwnd, statSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	CString statusTexts[] = {
			_T("Connecting to server..."),
			_T("Successfully connected to server."),
			_T("Authenticating on server..."),
			_T("Successfully authenticated on server."),
			_T("Getting message list from server..."),
			_T("Successfully got messages from server."),
			_T("Getting message UIDLs..."),
			_T("Successfully got messages UIDLs."),
			_T("Retrieving message %d of %d."),
			_T("Successfully retrieved messages."),
			_T("Deletting message %d of %d."),
			_T("Message successfully deleted."),
			_T("Sending NOOP..."),
			_T("NOOP successfully sent."),
			_T("Closing connection."),
			_T("Connection successfully closed.")
	};

	CString statusText;
	EmaiUint32 curMessage = 0;
	EmaiUint32 messageNumber = 0;
	switch (inOperationCode)
	{
	case EmaiPop3RetrievingMessage:
	case EmaiPop3DeletingMessage:
		inSession.GetUserData(Pop3CurrentMessage, &curMessage, numSize);
		inSession.GetUserData(Pop3MessageNumber, &messageNumber, numSize);
		statusText.Format(statusTexts[inOperationCode], curMessage + 1, messageNumber);
		break;
	default:
		ASSERT((inOperationCode >= EmaiPop3ConnectingToServer) && (inOperationCode <= EmaiPop3ConnectionClosed));
		statusText = statusTexts[inOperationCode];
		break;
	}

	wnd->SetWindowText(statusText);
	return EmaiErrNoErr;
}

EmaiError
Pop3Callback::OnDataReceived(
		Emai::Pop3Session& inSession,
		EmaiUint32 inReceivedSize,
		EmaiUint32 inTotalSize
	)
{
	HWND hwnd = NULL;
	EmaiUint32 perSize = sizeof(HWND);
	inSession.GetUserData(Pop3StatusText, &hwnd, perSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	int percent = 0;
	int curPercent = (100 * inReceivedSize) / inTotalSize;
	EmaiUint32 numSize = sizeof(EmaiUint32);
	inSession.GetUserData(Pop3CurrentPercent, &percent, numSize);
	if (percent != curPercent)
	{
		CString message;
		message.Format(_T("%d %%"), curPercent);
		wnd->SetWindowText(message);
		inSession.SetUserData(Pop3CurrentPercent, &curPercent, numSize);
	}
	return EmaiErrNoErr;
}

EmaiError
Pop3Callback::OnServerError(
		Emai::Pop3Session& inSession,
		EmaiConstCharPtr inServerReply,
		EmaiUint32 /*inReplyCode*/
	)
{
	HWND hwnd = NULL;
	EmaiUint32 wndSize = sizeof(HWND);
	inSession.GetUserData(Pop3PercentText, &hwnd, wndSize);
	CWnd* wnd = CWnd::FromHandle(hwnd);
	CString message;
	message.Format(_T("The server error encountered was: %S"), inServerReply);
	wnd->SetWindowText(message);
	return EmaiErrNoErr;
}
