/*
// File:		SmtpGUIDlg.cpp
//
// Version:		mail.it library 1.0
//
// Copyright:	(C) 2007 Dacons LLP.
*/

#include PCH_FILE_NAME
#include "SmtpGUI.hpp"
#include "SmtpGUIDlg.hpp"

#include "SmtpCallback.hpp"
#include <EmaiSmtp.hpp>
#include <MessageComposer.hpp>

#include "ErrorsDescriptions.hpp"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif

CSmtpGUIDlg::CSmtpGUIDlg(CWnd* pParent /*=NULL*/)
: CDialog(CSmtpGUIDlg::IDD, pParent)
	, mMailServer(_T("mail.server.net"))
	, mAccount(_T("your.account"))
	, mPassword(_T("password"))
	, mSenderAddress(_T("sender@mail.server.net"))
	, mRecAddress(_T("recipient@othermail.server.net"))
	, mMsgSubject(_T("Subject"))
	, mPlainText(_T("Message text"))
	, mHtmlText(_T("<html><body>Message text</body></html>"))
	, mStatusText(_T(""))
	, mPerStatus(_T(""))
	, mPort(EmaiDefaultSmtpPort)
	, mUseSsl(FALSE)
	, mAttachText(FALSE)
	, mAttachHTML(FALSE)
	, mLicenseKey(_T("Put your license key here."))
{
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);
}

void CSmtpGUIDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Text(pDX, IDC_EDIT_MAILSERVER, mMailServer);
	DDX_Text(pDX, IDC_EDIT_ACCOUNT, mAccount);
	DDX_Text(pDX, IDC_EDIT_PASSWORD, mPassword);
	DDX_Text(pDX, IDC_EDIT_SENDER_ADDRESS, mSenderAddress);
	DDX_Text(pDX, IDC_EDIT_RECIPIENT_ADDRESS, mRecAddress);
	DDX_Text(pDX, IDC_EDIT_SUBJECT, mMsgSubject);
	DDX_Text(pDX, IDC_EDIT_PLAIN_TEXT, mPlainText);
	DDX_Text(pDX, IDC_EDIT_HTML_TEXT, mHtmlText);
	DDX_Text(pDX, IDC_STATIC_STATUS, mStatusText);
	DDX_Text(pDX, IDC_STATIC_PER_STATUS, mPerStatus);
	DDX_Text(pDX, IDC_EDIT_PORT, mPort);
	DDX_Check(pDX, IDC_CHECK_SSL, mUseSsl);
	DDX_Control(pDX, IDC_COMBO_AUTH_TYPE, mAuthType);
	DDX_Control(pDX, IDC_ATTACHMENTS_LIST, mAttachmentsList);
	DDX_Control(pDX, IDC_EDIT_HTML_TEXT, mHtmlEdit);
	DDX_Check(pDX, IDC_CHECK_ATTACH_TEXT, mAttachText);
	DDX_Check(pDX, IDC_CHECK_ATTACH_HTML, mAttachHTML);
	DDX_Control(pDX, IDC_EDIT_PLAIN_TEXT, mPlainTextEdit);
	DDX_Control(pDX, IDC_STATIC_HELP_MSG, mHelpMessage);
	DDX_Text(pDX, IDC_EDIT_LICKEY, mLicenseKey);
}

BEGIN_MESSAGE_MAP(CSmtpGUIDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	//}}AFX_MSG_MAP
	ON_BN_CLICKED(IDC_BUTTON_SEND, OnBnClickedButtonSend)
	ON_BN_CLICKED(IDC_BUTTON_ADD, OnBnClickedButtonAdd)
	ON_BN_CLICKED(IDC_BUTTON_DELETE, OnBnClickedButtonDelete)
	ON_NOTIFY(LVN_ITEMACTIVATE, IDC_ATTACHMENTS_LIST, OnLvnItemActivateAttachmentsList)
	ON_BN_CLICKED(IDC_CHECK_ATTACH_TEXT, OnBnClickedCheckAttachText)
	ON_BN_CLICKED(IDC_CHECK_ATTACH_HTML, OnBnClickedCheckAttachHtml)
	ON_EN_SETFOCUS(IDC_EDIT_LICKEY, OnEnSetfocusEditLickey)
	ON_EN_KILLFOCUS(IDC_EDIT_LICKEY, OnEnKillfocusEditLickey)
END_MESSAGE_MAP()


// CSmtpGUIDlg message handlers

BOOL CSmtpGUIDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	mAuthType.SetCurSel(0);
	mAttachmentsList.InsertColumn(0, _T("Inline"), LVCFMT_CENTER, -1, 0);

	mAttachmentsList.InsertColumn(1, _T("Path to attachment"), LVCFMT_CENTER, -1, 1);
	mAttachmentsList.SetColumnWidth(0, LVSCW_AUTOSIZE_USEHEADER);
	mAttachmentsList.SetColumnWidth(1, LVSCW_AUTOSIZE_USEHEADER);

	mAttachmentsList.SetExtendedStyle(LVS_EX_CHECKBOXES | LVS_EX_FULLROWSELECT);


	return TRUE;  // return TRUE  unless you set the focus to a control
}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CSmtpGUIDlg::OnPaint() 
{
	if (IsIconic())
	{
		CPaintDC dc(this); // device context for painting

		SendMessage(WM_ICONERASEBKGND, reinterpret_cast<WPARAM>(dc.GetSafeHdc()), 0);

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
	}
	else
	{
		CDialog::OnPaint();
	}
}

// The system calls this function to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CSmtpGUIDlg::OnQueryDragIcon()
{
	return static_cast<HCURSOR>(m_hIcon);
}

void CSmtpGUIDlg::FormatError(Emai::Exception& inException)
{
	AfxMessageBox(errorsDescriptions[::abs(inException.GetErrorCode())]);
}

CString CSmtpGUIDlg::GetFile(void)
{
	// Chose file using file open dialog
	CFileDialog dlg(TRUE);
	if(dlg.DoModal() == IDOK)
	{
		return dlg.GetPathName();
	}
	return CString(_T(""));
}

void CSmtpGUIDlg::OnBnClickedButtonSend()
{
	UpdateData();
	if (EmaiRegisterLibrary(mLicenseKey) == EmaiErrNoErr)
	{
		try
		{
			mStatusText = _T("Composing message...");
			UpdateData(FALSE);
			/*Create the message from different parts*/
			MessageComposer messageComposer;
			if (mAttachText)
				messageComposer.SetUnicodeText(mPlainText);

			CPath fileName;
			CString filePath;
			CString id;
			CString left = mHtmlText.Mid(0, mHtmlText.Find(_T("</body>")));
			CString right = mHtmlText.Mid(mHtmlText.Find(_T("</body>")));
			CString htmlText = left;
			CString tmp;
			// Add attachments
			for (int i = 0; i <= mAttachmentsList.GetItemCount() - 1; i++)
			{
				fileName.m_strPath = mAttachmentsList.GetItemText(i, 1);
				filePath = fileName.m_strPath;
				fileName.StripPath();
				if (mAttachmentsList.GetCheck(i))
				{
					tmp.Format(_T("<img src=\"cid:%s\">"), fileName);
					if (htmlText.Find(tmp) < 0)
						htmlText.Append(tmp);

					messageComposer.AddInlineFromFile(fileName, filePath, fileName);
				}
				else
				{
					messageComposer.AddAttachmentFromFile(fileName, filePath);
				}
			}
			htmlText += right;
			if (mAttachHTML)
			{
				USES_CONVERSION;
				messageComposer.SetHtml(W2A(htmlText));
			}

			/*Add contact fields to the message*/
			Emai::ContactList contactTo;
			contactTo.AddContact(mRecAddress, NULL);
			Emai::ContactList contactFrom;
			contactFrom.AddContact(mSenderAddress, NULL);

			Emai::Message message = messageComposer.ComposeMessage(contactFrom, NULL, contactTo,
				NULL, NULL, mMsgSubject);

			/*Check if message is created successufully*/
			if (message != NULL)
			{
				/*Create the session*/
				SmtpCallback statusCallback;
				EmaiOptions options = EmaiNullOptions;
				if (mUseSsl) 
					options = EmaiSmtpUseSSLOption;
				Emai::SmtpSession smtpSession(statusCallback, 60, options);

				// Add IDC_STATIC_PER_STATUS handle as users data to change its text in statusCallback
				HWND hwnd = *GetDlgItem(IDC_STATIC_PER_STATUS);
				EmaiUint32 perc = 0;
				smtpSession.SetUserData(SmtpCurrentPercent, &perc, sizeof(EmaiUint32));
				smtpSession.SetUserData(SmtpPercentText, &hwnd, sizeof(HWND));
				hwnd = *GetDlgItem(IDC_STATIC_STATUS);
				smtpSession.SetUserData(SmtpStatusText, &hwnd, sizeof(HWND));

				/*Connect to the server*/
				smtpSession.Connect(mMailServer);

				/*Authenticate the session*/
				EmaiUint32 authTypes[] = {
					EmaiSMTPAuthAutomatic,
						EmaiSMTPAuthLogin,
						EmaiSMTPAuthPlain,
						EmaiSMTPAuthCramMD5,
						EmaiSMTPAuthNTLM
				};
				smtpSession.Authenticate(mAccount, mPassword, authTypes[mAuthType.GetCurSel()]);

				/*Send the message*/
				smtpSession.SendMessage(message);

				/*Disconnect from the server*/
				smtpSession.Disconnect();
				mPerStatus = _T("");
				mStatusText = _T("");
				UpdateData(FALSE);
			}
		}
		catch (Emai::Exception& exception)
		{
			FormatError(exception);
			mPerStatus = _T("");
			mStatusText = _T("");
			UpdateData(FALSE);
		}
	}
	else
		AfxMessageBox(_T("Licence key is invalid. Restart application and enter valid key."));
}

void CSmtpGUIDlg::OnBnClickedButtonAdd()
{
	UpdateData();
	// Add new attachment to attachment list
	CString file = GetFile();
	if (!file.IsEmpty())
	{
		mAttachmentsList.InsertItem(mAttachmentsList.GetItemCount(), _T(""));
		mAttachmentsList.SetItem(mAttachmentsList.GetItemCount() - 1, 1, LVIF_TEXT, file,
				0, 0, 0, 0);
	}
	UpdateData(FALSE);
}

void CSmtpGUIDlg::OnBnClickedButtonDelete()
{
	// Delete attachment from attachment list
	POSITION pos = mAttachmentsList.GetFirstSelectedItemPosition();
	while (pos)
	{
		int item = mAttachmentsList.GetNextSelectedItem(pos);
		mAttachmentsList.DeleteItem(item);
	}
	UpdateData(FALSE);
}


void CSmtpGUIDlg::OnLvnItemActivateAttachmentsList(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMITEMACTIVATE pNMIA = reinterpret_cast<LPNMITEMACTIVATE>(pNMHDR);
	if (mAttachHTML)
	{
		UpdateData();
		mAttachmentsList.SetCheck(pNMIA->iItem, TRUE);
		CPath fileName;
		fileName.m_strPath = mAttachmentsList.GetItemText(pNMIA->iItem, 1);
		fileName.StripPath();
		CString text;
		text.Format(_T("<img src=\"cid:%s\">"), fileName);
		mHtmlEdit.ReplaceSel(text);
	}
	else
		AfxMessageBox(_T("You can not add inline attachment without html message part."));

	*pResult = 0;
}

void CSmtpGUIDlg::OnBnClickedCheckAttachText()
{
	UpdateData();
	mPlainTextEdit.EnableWindow(mAttachText);
}

void CSmtpGUIDlg::OnBnClickedCheckAttachHtml()
{
	UpdateData();
	mHtmlEdit.EnableWindow(mAttachHTML);
}

void CSmtpGUIDlg::OnEnSetfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.CompareNoCase(_T("Put your license key here.")) == 0)
	{
		mLicenseKey = _T("");
		UpdateData(FALSE);
	}
}

void CSmtpGUIDlg::OnEnKillfocusEditLickey()
{
	UpdateData();
	if (mLicenseKey.IsEmpty())
	{
		mLicenseKey = _T("Put your license key here.");
		UpdateData(FALSE);
	}
}
